#include "include.h"
#include "log.h"
#include "socket.h"

static struct uloop_fd server;

static void server_cb(struct uloop_fd *fd, unsigned int events) {
	struct sockaddr_in6 addr;
	char buf[REQUEST_MAXLEN];
	char control[256];


	struct iovec iv = {
		.iov_base = buf,
		.iov_len = sizeof(buf) - 1
	};

	struct msghdr mh = {
		.msg_name = &addr,
		.msg_namelen = sizeof(addr),
		.msg_iov = &iv,
		.msg_iovlen = 1,
		.msg_control = control,
		.msg_controllen = sizeof(control)
	};

	if (recvmsg(server.fd, &mh, MSG_WAITALL) < 0) {
		log_info("socket: invalid package\n");
		return;
	}

	char addr_str[INET6_ADDRSTRLEN];
	inet_ntop(AF_INET6, &(addr.sin6_addr), addr_str, INET6_ADDRSTRLEN);
	log_info("socket:recv [%s]:%d - %s\n", addr_str, ntohs(addr.sin6_port), buf);


	if (sendto(server.fd, addr_str, sizeof(addr_str) - 1, 0, (struct sockaddr *) &addr, sizeof(addr)) < 0) {
		log_error("socket: could not send answer\n");
	}
	log_info("socket: send answer\n");
}

int socket_init(char *ifname) {
	server.cb = server_cb;
	server.fd = usock(USOCK_UDP | USOCK_SERVER | USOCK_IPV6ONLY | USOCK_NUMERIC, SOCKET_ADDR, SOCKET_PORT);
	if (server.fd < 0) {
		return 1;
	}
	struct in6_addr mgroup_addr;

	if (!inet_pton(AF_INET6, SOCKET_MADDR, &mgroup_addr)) {
		log_error("socket: invalid multicast group\n");
		return -1;
	}

	struct ipv6_mreq mreq;

	mreq.ipv6mr_multiaddr = mgroup_addr;
	mreq.ipv6mr_interface = if_nametoindex(ifname);

	if (setsockopt(server.fd, IPPROTO_IPV6, IPV6_JOIN_GROUP, &mreq, sizeof(mreq)) == -1) {
		log_error("socket: unable to join multicast group\n");
		return -1;
	}

	uloop_fd_add(&server, ULOOP_READ);

	return 0;
}

void socket_close() {

}
